<?php
namespace Datec\DatecContact\Controller;

/***************************************************************
 *  Copyright notice
 *
 *  (c) 2014
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 ***************************************************************/

use Datec\DatecContact\Domain\Model\Contact;
use Datec\DatecContact\Domain\Model\ContactMessage;
use Datec\DatecContact\Domain\Repository\ContactMessageRepository;
use Datec\DatecContact\Domain\Repository\ContactRepository;
use Datec\DatecContact\Domain\Repository\SubjectRepository;
use Datec\DatecContact\Services\MailService;
use TYPO3\CMS\Core\Messaging\FlashMessage;
use TYPO3\CMS\Core\Utility\GeneralUtility;
use TYPO3\CMS\Extbase\Domain\Model\FrontendUser;
use TYPO3\CMS\Extbase\Domain\Repository\FrontendUserRepository;
use TYPO3\CMS\Extbase\Mvc\Controller\ActionController;
use TYPO3\CMS\Extbase\Persistence\Generic\PersistenceManager;
use TYPO3\CMS\Extbase\Utility\LocalizationUtility;

/**
 * @package datec_contact
 * @license http://www.gnu.org/licenses/gpl.html GNU General Public License, version 3 or later
 */
class ContactFormController extends ActionController {
	
	/**
	 * $extKey
	 */
	protected $extKey;
	
	/**
	 * $feUser
	 */
	protected $feUser;
	
	/**
	 * @var Contact $contact
	 */
	protected $contact;
	
	/**
	 * @var ContactMessage $contactMessage
	 */
	protected $contactMessage;
	
	/**
	 * $contactRepository
	 *
	 * @var ContactRepository
	 * */
	protected $contactRepository;
	
	/**
	 * $contactMessageRepository
	 *
	 * @var ContactMessageRepository
	 * */
	protected $contactMessageRepository;
	
	/**
	 * $mailService
	 *
	 * @var MailService
	 * */
	protected $mailService;
	
	/**
	 * $feUserRepository
	 *
	 * @var FrontendUserRepository
	 * */
	protected $feUserRepository;
	
	/**
	 * $persistenceManager
	 *
	 * @var PersistenceManager
	 * */
	protected $persistenceManager;
	
	/**
	 * $subjectRepository
	 * 
	 * @var SubjectRepository
	 * */
	protected $subjectRepository;
	
	/**
	 * initialize current action
	 * @return void
	 */
	public function initializeAction() {
		$this->extKey = $this->request->getControllerExtensionKey();
		$this->feUser = $this->getFeUser();
		
	}

    /**
     * @param ContactMessageRepository $contactMessageRepository
     */
    public function injectContactMessageRepository(ContactMessageRepository $contactMessageRepository)
    {
        $this->contactMessageRepository = $contactMessageRepository;
    }

    /**
     * @param ContactRepository $contactRepository
     */
    public function injectContactRepository(ContactRepository $contactRepository)
    {
        $this->contactRepository = $contactRepository;
    }

    /**
     * @param FrontendUserRepository $feUserRepository
     */
    public function injectFeUserRepository(FrontendUserRepository $feUserRepository)
    {
        $this->feUserRepository = $feUserRepository;
    }

    /**
     * @param MailService $mailService
     */
    public function injectMailService(MailService $mailService)
    {
        $this->mailService = $mailService;
    }

    /**
     * @param PersistenceManager $persistenceManager
     */
    public function injectPersistenceManager(PersistenceManager $persistenceManager)
    {
        $this->persistenceManager = $persistenceManager;
    }

    /**
     * @param SubjectRepository $subjectRepository
     */
    public function injectSubjectRepository(SubjectRepository $subjectRepository)
    {
        $this->subjectRepository = $subjectRepository;
    }
	
	/**
	 * action showContactForm
	 *
	 * @return void
	 */
	public function showContactFormAction() {		
		$this->initializeContactMessage();
		$formFields = $this->getFormFields();		
		$subjectsResult = $this->subjectRepository->findAll();
		if($subjectsResult) {
			$subjects = $subjectsResult->toArray();
			if(!empty($subjects)) {
				$this->view->assign('subjects', $subjects);
				
			}
		}
		$this->view->assign('duplicant1', md5(GeneralUtility::generateRandomBytes(64)));
		$this->view->assign('formFields', $formFields);
		$this->view->assign('contactMessage', $this->contactMessage);
		$this->view->assign('pageId', $GLOBALS['TSFE']->id);
		$this->view->assign('settings', $this->settings);
	}

    /**
     * action createContactMessage
     *
     * @return void
     * @throws \TYPO3\CMS\Extbase\Persistence\Exception\IllegalObjectTypeException
     */
	public function createContactMessageAction() {
		$this->initializeContactMessage();
		$success = TRUE;

        if (isset($this->settings['storagePid']) && $this->settings['mail']['internMailFrom']) {
            if ($this->request->hasArgument('duplicant1') && $this->request->hasArgument('duplicant2')) { // spam protection values, they can only match when form submitted via 'DatecContact.createContactMessage()'
                $duplicant1 = $this->request->getArgument('duplicant1');
                $duplicant2 = $this->request->getArgument('duplicant2');

                if ($duplicant1 == $duplicant2) {
                    if ($this->contact) {
                        if (!$this->contact->getFeUser()) { // has no frontend user assigned? is public contact
                            $contactResult = $this->contactRepository->findByCredentials($this->contact->getEmail());

                            if (is_object($contactResult)) {
                                // we know this public user
                                if ($contactResult->isBlocked()) {
                                    // that badboy is not allowed here
                                    $this->addFlashMessage(LocalizationUtility::translate('tx_dateccontact.messages.contact.blocked_true', $this->extKey), '', FlashMessage::WARNING);
                                    $success = FALSE;
                                } else {
                                    // he's a'ight, use him
                                    $this->contact = $contactResult;
                                }
                            } else if (empty($contactResult)) {
                                // we don't know this public user, dont use the result, save a new
                                $this->contactRepository->add($this->contact);
                                $this->persistenceManager->persistAll();
                            } else {
                                $this->addFlashMessage(LocalizationUtility::translate('tx_dateccontact.errors.dbError', $this->extKey), '', FlashMessage::ERROR);
                                $success = FALSE;
                            }
                        }
                    }

                    if ($success) { // no problems with the contact?
                        $this->contactMessage->setContact($this->contact);

                        $this->contactMessageRepository->add($this->contactMessage);

                        $this->addFlashMessage(LocalizationUtility::translate('tx_dateccontact.messages.contactFormController.createContactMessage_success', $this->extKey), '', FlashMessage::OK);

                        $formFields = $this->getFormFields();
                        // user mail
                        $htmlMailUser = $this->mailService->generateContactUserMail($this->contact, $this->contactMessage, $formFields);
                        $this->mailService->sendExternMail(LocalizationUtility::translate('tx_dateccontact.mails.subjects.contactUserMail', $this->extKey), $htmlMailUser, '', $this->contact->getEmail(), $this->contact->getFirstName() . " " . $this->contact->getLastName());
                        // admin mail
                        if ($this->settings['mail']['internMailTo']) {
                            $htmlMailAdmin = $this->mailService->generateContactAdminMail($this->contact, $this->contactMessage, $formFields);
                            $this->mailService->sendInternMail(LocalizationUtility::translate('tx_dateccontact.mails.subjects.contactAdminMail', $this->extKey), $htmlMailAdmin);
                        }
                    }
                } else {
                    $this->addFlashMessage(LocalizationUtility::translate('tx_dateccontact.messages.spamProtection_failed', $this->extKey), '', FlashMessage::ERROR);
                }
            } else {
                $this->addFlashMessage(LocalizationUtility::translate('tx_dateccontact.messages.spamProtection_failed', $this->extKey), '', FlashMessage::ERROR);
            }
        } else {
            $this->addFlashMessage(LocalizationUtility::translate('tx_dateccontact.errors.configurationError',$this->extKey), '', FlashMessage::ERROR);
        }

		$this->view->assign('contactMessage', $this->contactMessage);
		$this->view->assign('pageId', $GLOBALS['TSFE']->id);
		$this->view->assign('settings', $this->settings);
		
	}
	
	/**
	 * look for current frontend user
	 *
	 * @return array|bool $feUser or boolean FALSE if no user found
	 */
	private function getFeUser() {
		if(isset($GLOBALS['TSFE']->loginUser)) {
			return $GLOBALS['TSFE']->fe_user->user;
		}
	
		return FALSE;
	}
	
	/**
	 * use TS configuration to determine used form fields
	 *
	 * @return array $formFields
	 */
	private function getFormFields() {
		$formFields = [];
		
		$formFieldsConfiguration = explode(',',$this->settings['contact']['formFields']);
		$requiredFormFieldsConfiguration = explode(',',$this->settings['contact']['requiredFormFields']);
		
		if (count($formFieldsConfiguration)) {
			foreach ($formFieldsConfiguration as $propertyNameRaw) {
				$propertyName = GeneralUtility::underscoredToLowerCamelCase($propertyNameRaw);
		
				if ($this->contact->_hasProperty($propertyName)) {
					$formFields[$propertyName]['propertyName'] = $propertyName;
					$formFields[$propertyName]['propertyNameRaw'] = $propertyNameRaw;
					$formFields[$propertyName]['required'] = in_array($propertyNameRaw, $requiredFormFieldsConfiguration);
				}
			}
		}
		
		if (!in_array('email', array_keys($formFields))) {
			$this->addFlashMessage(LocalizationUtility::translate('tx_dateccontact.errors.configurationError',$this->extKey), '', FlashMessage::ERROR);
		}
		
		return $formFields;
	}
	
	/**
	 * initialize a new contact message object and fill it with form data when provided,
	 * also sets the assigned contact (creator of message)
	 *
	 * @return void
	 */
	private function initializeContactMessage () {
		$this->contactMessage = new ContactMessage();
		$this->contact = new Contact();

        $this->contactMessage->setPid($this->settings['storagePid']);
        $this->contact->setPid($this->settings['storagePid']);

        if ($this->feUser) {
            $contactResult = $this->contactRepository->findByFeUser($this->feUser['uid']);

            if (is_object($contactResult)) {
                $this->contact = $contactResult;
            } else if (empty($contactResult)) {
                // none found, use this fe user as new creator
                /* @var FrontendUser $feUserResult */
                $feUserResult = $this->feUserRepository->findByUid($this->feUser['uid']);
                if (isset($feUserResult)) {
                    $this->contact->setFeUser($feUserResult);
                }
            }
        }

        if ($this->request->hasArgument('contactMessage')) {
            /* @var ContactMessage $formDataContactMessage */
            $formDataContactMessage = $this->request->getArgument('contactMessage');
            foreach ($formDataContactMessage as $propertyName => $propertyValue) {
                $this->contactMessage->_setProperty($propertyName, $propertyValue);
            }
        }

        if ($this->request->hasArgument('contact')) {
            $formDataContact = $this->request->getArgument('contact');
            /* @var Contact $formDataContact */
            foreach ($formDataContact as $propertyName => $propertyValue) {
                $this->contact->_setProperty($propertyName, $propertyValue);
            }
        }
				
		$this->contactMessage->setContact($this->contact);
		
	}
	
}