<?php
namespace Datec\DatecContact\Domain\Repository;

/***************************************************************
 *  Copyright notice
 *
 *  (c) 2014
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 ***************************************************************/

use Datec\DatecContact\Domain\Model\Contact;
use TYPO3\CMS\Extbase\Persistence\QueryInterface;
use TYPO3\CMS\Extbase\Persistence\QueryResultInterface;
use TYPO3\CMS\Extbase\Persistence\Repository;

/**
 * @package datec_contact
 * @license http://www.gnu.org/licenses/gpl.html GNU General Public License, version 3 or later
 */
class ContactRepository extends Repository {
	
	/**
	 * PR: this is not ideal yet, because the query funtions do not support conditional sorting,
	 * that means either build cutom querries or have this slightly off output:
	 * the current sorting can place posts published (starttime) before created (crdate) posts on the same day, even if the created one is newer by time
	 */
	protected $defaultOrderings = array(
		'crdate' => QueryInterface::ORDER_DESCENDING
	 );
	
	/**
	 * overrides default settings initialisation
	 * - we do not want to restrict the backend storage pages
	 */
	 public function initializeObject() {
	 	$query = $this->createQuery();	 	
        $defaultQuerySettings = $query->getQuerySettings();
        $defaultQuerySettings->setRespectStoragePage(FALSE);
        $this->setDefaultQuerySettings($defaultQuerySettings);
        unset($query);
	}
	
	/**
	 * Finds all objects
	 *
	 * @param boolean $ignoreEnableFields ignore enable fields in search
	 * @param array $enableFieldsToBeIgnored array of columns to ignore when $ignoreEnableFields == TRUE
	 * @param boolean $includeDeleted include deleted in search, defaults to FALSE  
	 * @param array $newOrder array of 'COLUMNNAME' => \TYPO3\CMS\Extbase\Persistence\QueryInterface::ORDER_*
	 * @return QueryResultInterface|array The matching object if found, otherwise NULL
	 */
	public function findAll($ignoreEnableFields = FALSE, $enableFieldsToBeIgnored = array(), $includeDeleted = FALSE, $newOrder = array()) {
		$query = $this->createQuery();
		$defaultQuerySettings = $query->getQuerySettings();
		$defaultQuerySettings->setIgnoreEnableFields($ignoreEnableFields);
		$defaultQuerySettings->setIncludeDeleted($includeDeleted);
		$defaultQuerySettings->setEnableFieldsToBeIgnored($enableFieldsToBeIgnored);
		if (!empty($newOrder)) {
			$this->defaultOrderings = $newOrder;
		}
		$this->setDefaultQuerySettings($defaultQuerySettings);
		return $query->execute();
	}
	
	/**
	 * Finds an object matching the given identifier.
	 *
	 * @param integer $uid The identifier of the object to find
	 * @param boolean $ignoreEnableFields ignore enable fields in search
	 * @param array $enableFieldsToBeIgnored array of columns to ignore when $ignoreEnableFields == TRUE
	 * @param boolean $includeDeleted include deleted in search, defaults to FALSE 
	 * @return object The matching object if found, otherwise NULL
	 */
	public function findByUid($uid, $ignoreEnableFields = FALSE, $enableFieldsToBeIgnored = array(), $includeDeleted = FALSE) {
		$query = $this->createQuery();
		$defaultQuerySettings = $query->getQuerySettings();
		$defaultQuerySettings->setIgnoreEnableFields($ignoreEnableFields);
		$defaultQuerySettings->setIncludeDeleted($includeDeleted);
		$defaultQuerySettings->setEnableFieldsToBeIgnored($enableFieldsToBeIgnored);
		$this->setDefaultQuerySettings($defaultQuerySettings);
		$object = $query->matching($query->equals('uid', $uid))->execute()->getFirst();
	
		return $object;
	}
	
	/**
	 * finds obect by id of frontend user
	 *
	 * @param integer $feUserId assigned frontend user identificator for this object
	 * @return Contact|bool $result
	 */
	public function findByFeUser($feUserId) {
		$result = FALSE;
		$constraints = array();
	
		$query = $this->createQuery();
	
		if ($feUserId) {
			$constraints[] = $query->equals('fe_user', $feUserId);
		}
	
		if (count($constraints)) {
			$result = $query->matching($query->logicalAnd($query->logicalAnd($constraints)))->execute()->getFirst();
		}
	
		unset($query);
	
		return $result;
	}
	
	/**
	 * finds obect by entered person information
	 * we use this to determine known public users
	 *
	 * @param string $email
	 * @return Contact|bool $result
	 */
	public function findByCredentials($email) {
		$result = FALSE;
		$constraints = array();
	
		$query = $this->createQuery();
	
		if ($email) {
			$constraints[] = $query->equals('email', $email);
		}
	
		if (count($constraints)) {
			$result = $query->matching($query->logicalAnd($query->logicalAnd($constraints)))->execute()->getFirst();
		}
	
		unset($query);
	
		return $result;
	}
	
}